[patch 14/20] chardev: GPIO for SCx200:

[Date Prev][Date Next][Thread Prev][Thread Next][Date Index][Thread Index]

 




Adds platform-device to (just introduced) driver, and uses it to
replace many printks with dev_dbg() etc.  This is because pdev was
retrofitted into the patchset. I could trivially merge this into previous patch ?

Signed-off-by:  Jim Cromie <jim.cromie@xxxxxxxxx>

diffstat patch.pdev-pc8736x
pc8736x_gpio.c | 78 ++++++++++++++++++++++++++++++++++----------------------
1 files changed, 47 insertions(+), 31 deletions(-)

---

diff -ruNp -X dontdiff -X exclude-diffs ab-13/drivers/char/pc8736x_gpio.c ab-14/drivers/char/pc8736x_gpio.c
--- ab-13/drivers/char/pc8736x_gpio.c	2006-06-02 13:38:39.000000000 -0600
+++ ab-14/drivers/char/pc8736x_gpio.c	2006-06-02 13:38:55.000000000 -0600
@@ -17,13 +17,14 @@
#include <linux/init.h>
#include <linux/ioport.h>
#include <linux/nsc_gpio.h>
+#include <linux/platform_device.h>
#include <asm/uaccess.h>
#include <asm/io.h>

-#define NAME "pc8736x_gpio"
+#define DEVNAME "pc8736x_gpio"

MODULE_AUTHOR("Jim Cromie <jim.cromie@xxxxxxxxx>");
-MODULE_DESCRIPTION("NatSemi SCx200 GPIO Pin Driver");
+MODULE_DESCRIPTION("NatSemi PC-8736x GPIO Pin Driver");
MODULE_LICENSE("GPL");

static int major = 0;		/* default to dynamic major */
@@ -43,6 +44,8 @@ static unsigned pc8736x_gpio_base;

#define SIO_CF1		0x21	/* chip config, bit0 is chip enable */

+#define PC8736X_GPIO_SIZE	16
+
#define SIO_UNIT_SEL	0x7	/* unit select reg */
#define SIO_UNIT_ACT	0x30	/* unit enable */
#define SIO_GPIO_UNIT	0x7	/* unit number of GPIO */
@@ -70,6 +73,8 @@ static int port_offset[] = { 0, 4, 8, 10
#define PORT_EVT_EN	2
#define PORT_EVT_STST	3

+static struct platform_device *pdev;	/* mostly for dev_*() support */
+
static inline void sio_outb(int addr, int val)
{
	outb_p(addr, sio_cmd);
@@ -155,9 +160,9 @@ static int pc8736x_gpio_get(unsigned min
	val >>= bit;
	val &= 1;

-	printk(KERN_INFO NAME ": _gpio_get(%d from %x bit %d) == val %d\n",
-	       minor, pc8736x_gpio_base + port_offset[port] + PORT_IN, bit,
-	       val);
+	dev_dbg(&pdev->dev, "_gpio_get(%d from %x bit %d) == val %d\n",
+		minor, pc8736x_gpio_base + port_offset[port] + PORT_IN, bit,
+		val);

	return val;
}
@@ -171,22 +176,21 @@ static void pc8736x_gpio_set(unsigned mi

	curval = inb_p(pc8736x_gpio_base + port_offset[port] + PORT_OUT);

-	printk(KERN_INFO NAME
-	       ": addr:%x cur:%x bit-pos:%d cur-bit:%x + new:%d -> bit-new:%d\n",
-	       pc8736x_gpio_base + port_offset[port] + PORT_OUT,
-	       curval, bit, (curval & ~(1 << bit)), val, (val << bit));
+	dev_dbg(&pdev->dev, "addr:%x cur:%x bit-pos:%d cur-bit:%x + new:%d -> bit-new:%d\n",
+		pc8736x_gpio_base + port_offset[port] + PORT_OUT,
+		curval, bit, (curval & ~(1 << bit)), val, (val << bit));

	val = (curval & ~(1 << bit)) | (val << bit);

-	printk(KERN_INFO NAME ": gpio_set(minor:%d port:%d bit:%d"
-	       ") %2x -> %2x\n", minor, port, bit, curval, val);
+	dev_dbg(&pdev->dev, "gpio_set(minor:%d port:%d bit:%d)"
+		" %2x -> %2x\n", minor, port, bit, curval, val);

	outb_p(val, pc8736x_gpio_base + port_offset[port] + PORT_OUT);

	curval = inb_p(pc8736x_gpio_base + port_offset[port] + PORT_OUT);
	val = inb_p(pc8736x_gpio_base + port_offset[port] + PORT_IN);

-	printk(KERN_INFO NAME ": wrote %x, read: %x\n", curval, val);
+	dev_dbg(&pdev->dev, "wrote %x, read: %x\n", curval, val);
}

static void pc8736x_gpio_set_high(unsigned index)
@@ -201,7 +205,7 @@ static void pc8736x_gpio_set_low(unsigne

static int pc8736x_gpio_current(unsigned index)
{
-	printk(KERN_WARNING NAME ": pc8736x_gpio_current unimplemented\n");
+	dev_warn(&pdev->dev, "pc8736x_gpio_current unimplemented\n");
	return 0;
}

@@ -229,7 +233,7 @@ static int pc8736x_gpio_open(struct inod
	unsigned m = iminor(inode);
	file->private_data = &pc8736x_access;

-	printk(KERN_NOTICE NAME " open %d\n", m);
+	dev_dbg(&pdev->dev, "open %d\n", m);

	if (m > 63)
		return -EINVAL;
@@ -237,20 +241,30 @@ static int pc8736x_gpio_open(struct inod
}

static struct file_operations pc8736x_gpio_fops = {
-	.owner = THIS_MODULE,
-	.open = pc8736x_gpio_open,
-	.write = nsc_gpio_write,
-	.read = nsc_gpio_read,
+	.owner	= THIS_MODULE,
+	.open	= pc8736x_gpio_open,
+	.write	= nsc_gpio_write,
+	.read	= nsc_gpio_read,
};

static int __init pc8736x_gpio_init(void)
{
	int r, rc;

-	printk(KERN_DEBUG NAME " initializing\n");
+        pdev = platform_device_alloc(DEVNAME, 0);
+        if (!pdev)
+                return -ENOMEM;
+
+        rc = platform_device_add(pdev);
+        if (rc) {
+                platform_device_put(pdev);
+                return -ENODEV;
+        }
+        dev_info(&pdev->dev, "NatSemi pc8736x GPIO Driver Initializing\n");

	if (!pc8736x_superio_present()) {
-		printk(KERN_ERR NAME ": no device found\n");
+		dev_err(&pdev->dev, "no device found\n");
+                platform_device_put(pdev);
		return -ENODEV;
	}

@@ -259,31 +273,33 @@ static int __init pc8736x_gpio_init(void
	 */
	rc = sio_inb(SIO_CF1);
	if (!(rc & 0x01)) {
-		printk(KERN_ERR NAME ": device not enabled\n");
+		dev_err(&pdev->dev, "Super-IO device not enabled\n");
		return -ENODEV;
	}
	device_select(SIO_GPIO_UNIT);
	if (!sio_inb(SIO_UNIT_ACT)) {
-		printk(KERN_ERR NAME ": GPIO unit not enabled\n");
+		dev_err(&pdev->dev, "GPIO LDN-unit not enabled\n");
		return -ENODEV;
	}

-	/* read GPIO unit base address */
+	/* read GPIO unit base address via the Super-IO port */
+
	pc8736x_gpio_base = (sio_inb(SIO_BASE_HADDR) << 8
			     | sio_inb(SIO_BASE_LADDR));

-	if (request_region(pc8736x_gpio_base, 16, NAME))
-		printk(KERN_INFO NAME ": GPIO ioport %x reserved\n",
-		       pc8736x_gpio_base);
+	if (request_region(pc8736x_gpio_base, PC8736X_GPIO_SIZE, DEVNAME))
+		dev_info(&pdev->dev, "GPIO ioport %x reserved\n",
+			 pc8736x_gpio_base);
+

-	r = register_chrdev(major, NAME, &pc8736x_gpio_fops);
+	r = register_chrdev(major, DEVNAME, &pc8736x_gpio_fops);
	if (r < 0) {
-		printk(KERN_ERR NAME ": unable to register character device\n");
+		dev_err(&pdev->dev, "unable to register character device\n");
		return r;
	}
	if (!major) {
		major = r;
-		printk(KERN_DEBUG NAME ": got dynamic major %d\n", major);
+		dev_dbg(&pdev->dev, ": got dynamic major %d\n", major);
	}

	return 0;
@@ -291,11 +307,11 @@ static int __init pc8736x_gpio_init(void

static void __exit pc8736x_gpio_cleanup(void)
{
-	printk(KERN_DEBUG NAME " cleanup\n");
+	dev_dbg(&pdev->dev, " cleanup\n");

	release_region(pc8736x_gpio_base, 16);

-	unregister_chrdev(major, NAME);
+	unregister_chrdev(major, DEVNAME);
}

module_init(pc8736x_gpio_init);



--
Kernelnewbies: Help each other learn about the Linux kernel.
Archive:       http://mail.nl.linux.org/kernelnewbies/
FAQ:           http://kernelnewbies.org/faq/


[Index of Archives]     [Newbies FAQ]     [Linux Kernel Mentors]     [Linux Kernel Development]     [IETF Annouce]     [Git]     [Networking]     [Security]     [Bugtraq]     [Yosemite]     [MIPS Linux]     [ARM Linux]     [Linux RAID]     [Linux SCSI]     [Linux ACPI]
  Powered by Linux